@@ -27,17 +27,19 @@
 #include <QStandardPaths>
 
 #include <KServiceTypeTrader>
-#include <KSycoca>
+#include <KPluginInfo>
 
 #include <Plasma/DataContainer>
+#include <KPackage/PackageLoader>
 
 #include "cachedprovider.h"
+#include "comicproviderkross.h"
 
 ComicEngine::ComicEngine(QObject* parent, const QVariantList& args)
     : Plasma::DataEngine(parent, args), mEmptySuffix(false)
 {
     setPollingInterval(0);
-    updateFactories();
+    loadProviders();
 }
 
 ComicEngine::~ComicEngine()
@@ -48,7 +50,6 @@ void ComicEngine::init()
 {
     connect(Solid::Networking::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)),
              this, SLOT(networkStatusChanged(Solid::Networking::Status)));
-    connect(KSycoca::self(), SIGNAL(databaseChanged(QStringList)), this, SLOT(sycocaUpdated(QStringList)));
 }
 
 void ComicEngine::networkStatusChanged(Solid::Networking::Status status)
@@ -59,41 +60,32 @@ void ComicEngine::networkStatusChanged(Solid::Networking::Status status)
     }
 }
 
-void ComicEngine::sycocaUpdated(const QStringList &changedResources)
+void ComicEngine::loadProviders()
 {
-    if (changedResources.contains(QLatin1String("services"))) {
-        updateFactories();
-    }
-}
-
-void ComicEngine::updateFactories()
-{
-    mFactories.clear();
+    mProviders.clear();
     removeAllData(QLatin1String("providers"));
-    KService::List services = KServiceTypeTrader::self()->query(QLatin1String("Plasma/Comic"));
-    Q_FOREACH (const KService::Ptr &service, services) {
-        mFactories.insert(service->property(QLatin1String("X-KDE-PluginInfo-Name"), QVariant::String).toString(),
-                           service);
-        if (service->isDeleted()) {
-            continue;
-        }
-        qDebug() << "ComicEngine::updateFactories()  service name=" << service->name();
+    auto comics = KPackage::PackageLoader::self()->listPackages("Plasma/Comic");
+    for (auto comic : comics) {
+        mProviders << comic.pluginId();
+
+        //qDebug() << "ComicEngine::loadProviders()  service name=" << comic.name();
         QStringList data;
-        data << service->name();
-        QFileInfo file(service->icon());
+        data << comic.name();
+        QFileInfo file(comic.iconName());
         if (file.isRelative()) {
-            data << QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString(QLatin1String("plasma-comic/%1.png")).arg(service->icon()));
+            data << QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString(QLatin1String("plasma-comic/%1.png")).arg(comic.iconName()));
         } else {
-            data << service->icon();
+            data << comic.iconName();
         }
-        setData(QLatin1String("providers"), service->property(QLatin1String("X-KDE-PluginInfo-Name"), QVariant::String).toString(), data);
+        setData(QLatin1String("providers"), comic.pluginId(), data);
     }
+    forceImmediateUpdateOfAllVisualizations();
 }
 
 bool ComicEngine::updateSourceEvent(const QString &identifier)
 {
     if (identifier == QLatin1String("providers")) {
-        updateFactories();
+        loadProviders();
         return true;
     } else if (identifier.startsWith(QLatin1String("setting_maxComicLimit:"))) {
         bool worked;
@@ -125,15 +117,15 @@ bool ComicEngine::updateSourceEvent(const QString &identifier)
         //: are mandatory
         if (parts.count() < 2) {
             setData(identifier, QLatin1String("Error"), true);
-            qCritical() << "Less than two arguments specified.";
+            qWarning() << "Less than two arguments specified.";
             return false;
         }
-        if (!mFactories.contains(parts[0])) {
+        if (!mProviders.contains(parts[0])) {
             // User might have installed more from GHNS
-            updateFactories();
-            if (!mFactories.contains(parts[0])) {
+            loadProviders();
+            if (!mProviders.contains(parts[0])) {
                 setData(identifier, QLatin1String("Error"), true);
-                qCritical() << identifier << "comic plugin does not seem to be installed.";
+                qWarning() << identifier << "comic plugin does not seem to be installed.";
                 return false;
             }
         }
@@ -146,18 +138,19 @@ bool ComicEngine::updateSourceEvent(const QString &identifier)
             setData(identifier, QLatin1String("Error automatically fixable"), true);
             setData(identifier, QLatin1String("Identifier"), identifier);
             setData(identifier, QLatin1String("Previous identifier suffix"), lastCachedIdentifier(identifier));
-            qWarning() << "No connection.";
+            qDebug() << "No connection.";
             return true;
         }
 
-        const KService::Ptr service = mFactories[parts[0]];
+        KPackage::Package pkg = KPackage::PackageLoader::self()->loadPackage("Plasma/Comic", parts[0]);
 
         bool isCurrentComic = parts[1].isEmpty();
 
         QVariantList args;
         ComicProvider *provider = 0;
 
-        const QString type = service->property(QLatin1String("X-KDE-PlasmaComicProvider-SuffixType"), QVariant::String).toString();
+        //const QString type = service->property(QLatin1String("X-KDE-PlasmaComicProvider-SuffixType"), QVariant::String).toString();
+        const QString type = pkg.metadata().value("X-KDE-PlasmaComicProvider-SuffixType");
         if (type == QLatin1String("Date")) {
             QDate date = QDate::fromString(parts[1], Qt::ISODate);
             if (!date.isValid())
@@ -169,12 +162,12 @@ bool ComicEngine::updateSourceEvent(const QString &identifier)
         } else if (type == QLatin1String("String")) {
             args << QLatin1String("String") << parts[1];
         }
-        args << service->storageId();
+        args << QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("plasma/comics/") + parts[0] + QLatin1String("/metadata.desktop"));
 
-        provider = service->createInstance<ComicProvider>(this, args);
+        //provider = service->createInstance<ComicProvider>(this, args);
+        provider = new ComicProviderKross(this, args);
         if (!provider) {
             setData(identifier, QLatin1String("Error"), true);
-            qWarning() << identifier << "plugin could be created.";
             return false;
         }
         provider->setIsCurrent(isCurrentComic);
